/*************************************************************************
* Dixit - Copyright (C) 2004-2010 Octavian Procopiuc                     *
* Contact: <oprocopiuc@sourceforge.net>                                  *
* Homepage: <http://dixit.sourceforge.net>                               *
*                                                                        *
* This program is free software: you can redistribute it and/or modify   *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation, either version 3 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* This program is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
*                                                                        *
* FILENAME       : mainwindow.h                                          *
* REVISION       : 2                                                     *
* COMMENTARY     : Header file for MainWindow Qt4 class                  *
* NAMING CONVENTIONS:    a_   MainWindow Qt4 action                      *
*                        m_   MainWindow member variable                 *
*                        w_   MainWindow member Qt4 window widget        *
*************************************************************************/

#ifndef MAINWINDOW_H                               /* duplication check */
#define MAINWINDOW_H

#include "depot.h"
#include <QDir>
#include <QHttp>
#include <QHttpResponseHeader>
#include <QListWidget>
#include <QMainWindow>
#include <QTimer>
#include <QUrl>

class QAction;
class QComboBox;
class QDir;
class QHttp;
class QHttpResponseHeader;
class QLabel;
class QLineEdit;
class QListWidget;
class QSplitter;
class QTextBrowser;
class QTimer;
class QUrl;

enum UpdateErrors {
  UPDATE_ERR_NO_ERROR,
  UPDATE_ERR_NETWORK,
  UPDATE_ERR_SERVER,
  UPDATE_ERR_DB_READ,
  UPDATE_ERR_DB_WRITE,
  UPDATE_ERR_CONTENT,
  UPDATE_ERR_STREAM,
  UPDATE_ERR_TIMEOUT,
  UPDATE_ERR_NOT_DONE,
  UPDATE_ERR_ABORTED
};


class MainWindow : public QMainWindow
{
  Q_OBJECT

public:
  MainWindow(void);
  ~MainWindow(void);
  int dbCreateIndex(void);

signals:
  void moreData(const QHttpResponseHeader &);

protected:
  QThread *m_pthread;

private slots:
  void abortUpdate(void);
  void anchorClicked(const QUrl &link);
  void backwardInHistory(void);
  void changeFont(void);
  void checkHttpResponseHeader(const QHttpResponseHeader &rh);
  void closeConnection(bool done);
  void connectionTimeout(void);
  void forwardInHistory(void);
  void getDexonlineData(const QHttpResponseHeader &rh);
  void help(void);
  void makeFontBigger(void);
  void makeFontSmaller(void);
  void print(void);
  void search(void);
  void setDefaultDictionary(int id);
  void showDictionary(int id);
  void update(void);
  void wordClickedInHistoryBox(const QString &word);

private:
#if QT_VERSION < 0x040500
  void     appendToByteArray(QByteArray &ba, const char  *buf, uint bufsz);
#endif
  void     clearBrowser(void);
  void     createIndependentActions(void);
  void     createMainWindow(void);
  void     createStatusBar(void);
  void     createToolBar(void);
  void     findDefinitions(const QString &word);
  void     firstPage(bool flgUpdate);
  void     genericUpdate(void);
  void     htmlizeDefBody(QString &html, const char *in);
  void     inflateData(QByteArray &idata, char *zdata, int zdatasz, bool init_only);
  void     insertInSearchBox(const QString &word);
  void     loadSettings(void);
  void     logInfo(const QString msg);
  void     processWord(const QString &word);
  void     saveSettings(void);
  void     setBrowserHtml(const QString &str, bool prs = true);
  void     setFocusToSearchBox(void);
  bool     dbClose(void);
  bool     dbInit(void);
  bool     dbKeyExists(const char *key);
  bool     dbOpenRead(void);
  bool     dbOpenWrite(void);
  bool     dbPut(const char *key, const char *data);
  char*    dbGet(const char *key, bool *ok);
  int      dbSaveXMLData(const QString &xmlData, UpdateErrors *updateError, bool *abortHttp);
  int      LevenshteinDistance(const QString &source, const QString &target, int maxdist);
  QString  formatRoDate(int seconds, bool hour = false);
  QString  dbResults(const QString &word);
  QString  getDictionaryName(const QString &shortName);
  QString  htmlizeXMLDefinition(const QString &data);
  QString  processCommands(const QString &cmd);
  QString  loadFile(const QString &fileName);

  bool            m_activeConnection;
  bool            m_autoUpdate;
  bool            m_displayDate;
  bool            m_displayDictionary;
  bool            m_displayVolunteer;
  bool            m_logInfo;
  char            m_dbFileName[128];
  char            m_lastUpdateTimeKey[16];
  int             m_lastUpdateTime;
  int             m_lastUpdateTimePending;
  int             m_defaultDictionary;
  int             m_newDefCount;
  int             m_newDefCountPending;
  int             m_totalDefinitions;
  UpdateErrors    m_updateError;
  QString         m_strBrowserHtml;
  QString         m_strCurrentWord;
  QStringList     m_lstAllWords;
  QStringList     m_lstBackwardWords;
  QStringList     m_lstForwardWords;
  QStringList     m_lstNewDefinitions;
  QTimer          m_connectionTimer;
  QHttp           *m_http;
  DEPOT           *m_dbHandle;

  // widgets
  QComboBox       *w_dictionaryComboBox;
  QListWidget     *w_historyListBox;
  QLineEdit       *w_searchBox;
  QTextBrowser    *w_browser;
  QToolBar        *w_toolBar;

  // actions
  QAction         *a_next;
  QAction         *a_prev;
};

#endif                                             /* duplication check */

/* END OF FILE */


